import {useEffect, useRef} from "react";

export type Modify<T, R> = Omit<T, keyof R> & R;

/**
 * Merge multiple class names to one full class name.
 * @param className Class names.
 */
export function classes(...className: (string|null|undefined|false)[]): string
{
	return className.filter((className) => !!className).join(" ");
}

export function formatDate(date: Date): string
{
	return ((date.getDate() < 10 ? "0" : "") + date.getDate()) + "/" + (((date.getMonth() + 1) < 10 ? "0" : "") + (date.getMonth() + 1)) + "/" + date.getFullYear();
}

export function formatTime(date: Date): string
{
	return (date.getHours() < 10 ? "0" : "") + date.getHours() + ":" + (date.getMinutes() < 10 ? "0" : "") + date.getMinutes();
}

/**
 * Normalize a given string for searching.
 * @param str The string to normalize.
 */
export function normalizeString(str: string): string
{
	return str
		? str.toLowerCase?.().normalize?.("NFD")
			.replace?.(/[\u0300-\u036f]/g, "")
		: "";
}

/**
 * Get the previous value of a given value.
 * @param currentValue The current value.
 */
export function usePreviousValue<T>(currentValue: T): T
{
	// Get the reference to the previous value.
	const ref = useRef<T>();

	// If the value has changed, saving it in the reference after rendering.
	useEffect(() => {
		ref.current = currentValue;
	}, [currentValue]);

	// Get the previous value.
	return ref?.current ?? undefined;
}
